/*

Production system

2018/11/06

*/


:- dynamic if/1, then/2.


:- op(10, xfy, then).
:- op(100, fx, if).


:- op(1000, xfy, and).
:- op(1100, xfy, or).

and(X, Y):-
	call(X), call(Y).

or(X, Y):-
	call(X); call(Y).




% Inference engine

psys :-
	repeat,
		conflict_set(Set),
		process_conflict_set(Set), !.

process_conflict_set([]):- !.
process_conflict_set(Set):-
	selected_action(Set, Action),
	perform_action(Action),
	!,
	fail.

conflict_set(Set):-
	findall(rule(Cond, Act), (rule(Cond, Act), satisfied(Cond)), Set).

% simple minded conflict resolution strategy
selected_action([rule(_, Act)|_], Act).


% simple, but maybe sufficient, condition evaluator
satisfied(Cond):- call(Cond).

% simple, but maybe sufficient, action executer
perform_action(Action):- call(Action).


rule(A, B):- if A then B.



% USEFUL STUFF
% What is the next list of actions to be executed?
selected_action_list(AList):-
	conflict_set(Rules),
	selected_action(Rules, Action),
	sequence_to_list(Action, AList).

% This predicate, with the dubious name sequence_to_list/2
% converts a sequnce of comma separated terms into a list
% e.g., (act1(v1), act2(v21, v22, v23), act3) --> [act1(v1), act2(v21, v22, v23), act3]
sequence_to_list((A1, A2), List):-
	!,
	sequence_to_list(A1, L1),
	sequence_to_list(A2, L2),
	append(L1, L2, List).
sequence_to_list(A, [A]).




%
% clearMemory(FactsPredicate):	Removes all clauses of the predicates specified
%				in the list maintained in the fact FactsPredicate
%
clearMemory(FactsPredicate) :-
	FACTS =.. [FactsPredicate, Facts],
	call(FACTS),
	clearFacts(Facts).
clearFacts([P/N | Fs]):-
	functor(Fact, P, N),
	retractall(Fact),
	clearFacts(Fs).
clearFacts([]).


list_rubbish(RubbishHoldingPred):-
	RUBBISHHOLDINGPRED =.. [RubbishHoldingPred, Rubbish],
	call(RUBBISHHOLDINGPRED),
	list_facts(Rubbish).


list_facts([P/N|Rubbish]):-
	functor(Fact, P, N),
	print_matching_facts(Fact),
	list_facts(Rubbish).
list_facts([]).


print_matching_facts(Fact):-
	call(Fact),
	write(Fact), nl,
	fail.
print_matching_facts(_).



writelist([X|Rest]):-
	write(X),
	writelist(Rest).
writelist([]).


























